import {
  AutocompletePicklistItem
} from '../interface/autocomplete-item';
import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { EmployeePicklistService } from './employee-typeahead-picklist.service';
import {
  GenericTypeaheadComponent
} from '../generic-typeahead/generic-typeahead.component';
import { filter, map, startWith } from 'rxjs/operators';


/** Picklist specifically for employees. */
@Component({
  selector: 'sis-employee-typeahead-picklist',
  styleUrls: ['./employee-typeahead-picklist.component.scss'],
  templateUrl: './employee-typeahead-picklist.component.html',
})
export class EmployeeTypeaheadPicklistComponent
extends GenericTypeaheadComponent
implements OnInit {

  // TODO add input flag to only show active employees
  // TODO add input flag to show all except terminated employees

  // Prevent values from being chosen if they aren't in the picklist.
  // Overridden from parent class.
  allowValuesNotInPicklist = false;

  // Minimum number of characters that must be entered in the input field
  // before filtering starts.
  minFilterLength = 3;

  // List of available options that the user can choose from.
  // This overrides the value from the parent to prevent employee
  // data from being sent in by the parent control. This data should
  // always be retrieved from the employee picker service.
  picklistItems: Map<string, AutocompletePicklistItem>;

  // Accessor for retrieving the text entered into the input box.
  @ViewChild("selectedValueTextField") selectedValueTextField: ElementRef;

  constructor(private employeeService: EmployeePicklistService) {
    super();
    this.allowValuesNotInPicklist = false;
    // TODO Don't allow picklistItems to be passed in as an input.
    //      Instead, pull this data directly from employee picklist service.
  }

  /** Filter list choices based on text entered in the input box.
   * Override the parent class because the filtering for this picklist
   * doesn't need to be passed in dynamically.
   * @param formGroupValue Form group selected value.
   * @returns List of filtered items.
   */
  protected filter(formGroupValue: any): AutocompletePicklistItem[] {
    const text = this.getInputText(formGroupValue);
    if (!text) {
      return Array.from(this.picklistItems.values());
    }
    const filterValue = text.toLowerCase();
    const filtered: AutocompletePicklistItem[] = [];
    this.picklistItems.forEach((value, key) => {
      if (key.includes(filterValue)) {
         filtered.push(value);
      }
    });
    return filtered;
  }

  /** Setup the component. */
  ngOnInit() {
    this.employeeService.getEmployees().subscribe(
      (employees: Map<string, AutocompletePicklistItem>) => {
        this.picklistItems = employees;
      }
    );
    this.filteredItems = this.formGroup.valueChanges
      .pipe(
        startWith(''),
        filter((): boolean => {
          return !(
            this.selectedValueTextField.nativeElement.value &&
            this.selectedValueTextField.nativeElement.value.length >
            this.minFilterLength
          );
        }),
        map((value: any): AutocompletePicklistItem[] => {
          return this.filter(value);
        })
      );
  }

}
