import {
  EmployeePicklistItem
} from '../interface/employee-picklist-item';


/** A wrapper for data items displayed in the employee piclist. */
export class Employee
implements EmployeePicklistItem {

  // Unique string used for typeahead filtering.
  searchableName = '';

  constructor(
    public contact: any,
    public employee: any,
    public jobTitle: any
  ) {
    this.generateHashKey();
  }

  /** Filter values that are displayed in the picklist.
   * @param text User input to use for filtering.
   * @returns True if the item should be displayed.
   */
  filterCustom(text: string): boolean {
    return this.searchableName.includes(text);
  }

  /** Get the employee number.
   * @returns Employee number.
   */
  getEmployeeNumber(): string {
    return this.employee.employeeNumber;
  }

  /** Get the employee's current employment status.
   * @returns Employment status.
   */
  getEmploymentStatus(): string {
    return this.employee.employmentStatus;
  }

  /** Generate a hash key to represent this record. */
  protected generateHashKey(): void {
    this.searchableName =
      (
        String(this.employee.employeeNumber) +
        String(this.employee.firstName) +
        String(this.employee.lastName)
      ).toLowerCase();
  }

  /** Get the employee's current job title.
   * @returns Job title.
   */
  getJobTitle(): string {
    return this.jobTitle.name;
  }

  /** Get the string to display for an item when it is selected.
   * @returns Text to display in the input box.
   */
  getName(): string {
    return `${this.employee.firstName} + ${this.employee.lastName}`;
  }
}
