import {
  AutocompletePicklistItem
} from '../../projects/lib-ng-picklist/src/lib/interface/autocomplete-item';
import { Component } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { DemoData, Produce } from './demo-data';


/** Main class for the wrapper application. */
@Component({
  selector: 'app-root',
  styleUrls: ['./app.component.css'],
  templateUrl: './app.component.html',
})
export class AppComponent {

  demoData: DemoData;
  demoForm = this.fb.group({
    employee: ['', [Validators.required]],
    fruit: ['', [Validators.required, Produce.validateNameForm()]],
    vegetable: ['', [Produce.validateNameForm()]],
  });
  filteredProduce: Observable<any[]>;
  fruits: Map<string, AutocompletePicklistItem>;
  selectedVegetable: Produce;
  vegetables: Map<string, AutocompletePicklistItem>;

  constructor(private fb: FormBuilder) {
    this.demoData = new DemoData();
    this.fruits = this.demoData.fruits;
    this.vegetables = this.demoData.vegetables;
    this.selectedVegetable = new Produce(null, null);
  }

  /** Handle a selection change on the fruit element.
   * @param fruit Selected fruit.
   */
  onSelectFruit(fruit: AutocompletePicklistItem | string): void {
    if (fruit && typeof fruit !== 'string') {
      console.log(
        `Selected fruit: ${fruit.getName()}`
      );
    }
    else {
      console.log(`Selected fruit: ${fruit}`);
    }
  }

  /** Handle a selection change on the vegetable element.
   * @param vegetable Selected vegetable.
   */
  onSelectVegetable(vegetable: AutocompletePicklistItem | string): void {
    if (vegetable && typeof vegetable !== 'string') {
      console.log(
        `Selected vegetable: ${vegetable.getName()} `
      );
    }
    else {
      console.log(`Selected vegetable: ${vegetable}`);
    }
  }
}
