import {
  AutocompletePicklistItem
} from '../../projects/lib-ng-picklist/src/lib/interface/autocomplete-item';
import { AbstractControl, ValidatorFn } from '@angular/forms';

// Disable magic numbers since all values are for testing.
/* tslint:disable:no-magic-numbers */


const fruitNames = [
  'Apple',
  'Cherry',
  'Melon',
  'Peach',
  'Pear',
];
const vegetableNames = [
  'Broccoli',
  'Cabbage',
  'Lettuce',
  'Onion',
  'Squash',
  'Tomato',
];


/** Demo data for displaying produce items. */
export class Produce
implements AutocompletePicklistItem {

  constructor(public id: number, public name: string) {
    // No action.
  }

  // Use default filtering provided by the autocomplete picklist component.
  filterCustom: null;

  /** Get the name to display for a job title.
   * @returns Job title name.
   */
  getName(): string {
    return this.name;
  }

  /** Set the name of the produce.
   * @param name Produce name.
   */
  setName(name: string): void {
    if (Produce.validateName(name)) {
      this.name = name;
    }
  }

  /** Validate the name field.
   * @param name Produce name.
   * @returns True if valid.
   */
  static validateName(name: string): boolean {
    if (name) {
      if (name.length > 10) {
        console.log(`Name length of ${name.length} is out of bounds.`);
        return false;
      }
    }
    return true;
  }

  /** Custom Angular form control validation wrapper for name field.
   * @returns Validator function to be used in an Angular form control.
   */
  static validateNameForm(): ValidatorFn {
    return (control: AbstractControl): {[key: string]: any} | null => {
      const name =
        (control.value && control.value.getName) ?
        control.value.getName() :
        control.value;
      if (Produce.validateName(name)) {
        return null;
      }
      return {undefinedName: {value: control.value}};
    };
  }
}


/** Data used on demo page for the components in this library. */
export class DemoData {

  fruits: Map<string, Produce>;
  vegetables: Map<string, Produce>;

  constructor() {
    this.fruits = new Map<string, Produce>();
    this.vegetables = new Map<string, Produce>();
    let i = 0;
    for (const name of fruitNames) {
      this.fruits.set(name.toLowerCase(), new Produce(i, name));
      i++;
    }
    i = 0;
    for (const name of vegetableNames) {
      this.vegetables.set(name.toLowerCase(), new Produce(i, name));
      i++;
    }
  }
}
